@@ -342,6 +342,10 @@ def GetApp(appName=None, *a, OtherAdminStuff=None, **k):
     if secretKey is None:
         secretKey = GetMachineUniqueID()
 
+    projectPath = k.pop('projectPath', '')  # for pipenv file references within virtualenv
+    global PROJECT_PATH
+    PROJECT_PATH = projectPath
+
     app = Flask(
         appName,
         *a,
@@ -359,6 +363,7 @@ def GetApp(appName=None, *a, OtherAdminStuff=None, **k):
     app.domainName = domainName
 
     @app.route('/echo')
+    @VerifyAdmin
     def Echo():
         d = {}
         for k in dir(request):
@@ -367,6 +372,7 @@ def GetApp(appName=None, *a, OtherAdminStuff=None, **k):
 
         return jsonify(d)
 
+    # Flask-ApScheduler
     scheduler = flask_apscheduler.APScheduler()
     scheduler.init_app(app)
     # Store jobs in the database so they persist between restarts
@@ -679,7 +685,7 @@ def VerifyAdmin(func):
             flash('You are not an admin', 'danger')
             return redirect('/login')
 
-    return VerifyLoginWrapper
+    return VerifyAdminWrapper
 
 
 MenuOptionClass = namedtuple('MenuOptionClass', ['title', 'url', 'active'])
@@ -713,15 +719,13 @@ PROJECT_PATH = ''
 
 def SetupRegisterAndLoginPageWithPassword(
         app,
-        mainTemplate,  # should be like mainTemplate='main.html',
-        templatesPath=None,  # should be like templatesPath='/home/grant/signage/templates',
+        mainTemplate,  # should be like mainTemplate='main.html', all templates should be in the PROJECT_PATH/templates
         redirectSuccess=None,
         callbackFailedLogin=None,
         callbackNewUserRegistered=None,
         loginTemplate=None,
         registerTemplate=None,
         forgotTemplate=None,
-        projectPath=None,  # added for pipenv
 ):
     '''
     Use this function with the @VerifyLogin decorator to simplify login auth
@@ -729,23 +733,12 @@ def SetupRegisterAndLoginPageWithPassword(
     form should have at least two elements
 
     '''
-    global PROJECT_PATH
-    PROJECT_PATH = projectPath
-
-    if 'win' in sys.platform:
-        mainPath = _PathlibPath(os.path.dirname(sys.modules['__main__'].__file__))
-        TEMPLATES_PATH = PathString(mainPath / 'templates')
-    else:
-        if templatesPath is None:
-            TEMPLATES_PATH = PathString('/templates')
-        else:
-            TEMPLATES_PATH = PathString(templatesPath)
 
     if loginTemplate is None:
         templateName = 'autogen_login.html'
         loginTemplate = templateName
 
-        thisTemplatePath = PathString(TEMPLATES_PATH + '/' + templateName)
+        thisTemplatePath = PathString('templates/' + templateName)
         print('thisTemplatePath=', thisTemplatePath)
         if not os.path.exists(thisTemplatePath):
             with open(thisTemplatePath, mode='wt') as file:
@@ -779,7 +772,7 @@ def SetupRegisterAndLoginPageWithPassword(
         templateName = 'autogen_register.html'
         registerTemplate = templateName
 
-        thisTemplatePath = PathString(TEMPLATES_PATH + '/' + templateName)
+        thisTemplatePath = PathString('templates/' + templateName)
         if not os.path.exists(thisTemplatePath):
             with open(thisTemplatePath, mode='wt') as file:
                 file.write('''
@@ -958,7 +951,7 @@ def SetupRegisterAndLoginPageWithPassword(
         templateName = 'autogen_forgot.html'
         forgotTemplate = templateName
 
-        thisTemplatePath = PathString(TEMPLATES_PATH + '/' + templateName)
+        thisTemplatePath = PathString('templates/' + templateName)
         if not os.path.exists(thisTemplatePath):
             with open(thisTemplatePath, mode='wt') as file:
                 file.write('''
@@ -1120,52 +1113,22 @@ def GetNumOfJobs():
     return len(GetJobs())
 
 
-def AddJob(callback, *args, **kwargs):
+def AddJob(callback, *args, name=None, misfire_grace_time=10, **kwargs):
     print('flask_tools.AddJob(callback={}, args={}, kwargs={})'.format(callback, args, kwargs))
-    jobID = ScheduleJob(
-        datetime.datetime.now() + datetime.timedelta(seconds=1),
-        callback,
-        *args,
-        **kwargs,
+    Log('AddJob', callback, args, kwargs, name)
+    jobID = GetRandomID(8)
+    app.apscheduler.add_job(
+        id=jobID,
+        name=name,
+        func=callback,
+        args=args,
+        kwargs=kwargs,
+        misfire_grace_time=misfire_grace_time,  # seconds
+        trigger='date',  # needed when running on ubuntu
     )
     return jobID
 
 
-class LimitedSizeDict(OrderedDict):
-    def __init__(self, *args, **kwds):
-        self.size_limit = kwds.pop("size_limit", None)
-        OrderedDict.__init__(self, *args, **kwds)
-        self._check_size_limit()
-
-    def __setitem__(self, key, value):
-        OrderedDict.__setitem__(self, key, value)
-        self._check_size_limit()
-
-    def _check_size_limit(self):
-        if self.size_limit is not None:
-            while len(self) > self.size_limit:
-                self.popitem(last=False)
-
-
-jobProgress = LimitedSizeDict(size_limit=100)
-
-
-def LogJobProgress(ID, progress):
-    jobProgress[ID] = progress
-
-
-def GetJobProgress(ID):
-    print('jobProgress=', jobProgress)
-    return jobProgress.get(ID, 'Unknown')
-
-
-def IsJobComplete(ID):
-    for job in GetJobs():
-        if ID == job.id:
-            return False
-    return True
-
-
 def PathString(path):
     if 'win' in sys.platform:
         path = _PathlibPath(path)
@@ -1173,22 +1136,33 @@ def PathString(path):
             return str(path)[1:]
         else:
             return str(path)
+
     else:  # linux
+        Log('sys.platform=', sys.platform)
         mainPath = _PathlibPath(os.path.dirname(sys.modules['__main__'].__file__)).parent
+        Log('mainPath=', mainPath)
 
         if 'app/.heroku' in str(mainPath):
-            # for heroku
+            # for heroku, note: Heroku files are ephemeral
             if str(path).startswith('/'):
                 return str(path)[1:]
             else:
                 return str(path)
+
+        elif 'virtualenv' in __file__:
+            Log('1153 virtualenv')
+            # when using pipenv
+            projPath = _PathlibPath(PROJECT_PATH)
+            Log('projPath=', projPath)
+            ret = projPath / path
+            Log('ret=', ret)
+            ret = str(ret)
+            Log('str(ret)=', ret)
+            return ret
+
         else:
-            if 'virtualenv' in str(mainPath):
-                # when using pipenv
-                return str(_PathlibPath(PROJECT_PATH) / path)[:1]
-            else:
-                newPath = mainPath / path
-                return str(newPath)[1:]
+            newPath = mainPath / path
+            return str(newPath)[1:]
 
 
 class File:
@@ -1499,6 +1473,7 @@ def ScheduleJob(dt, callback, *args, misfire_grace_time=60, name=None, **kwargs)
     :return:
     '''
     print('SchedulJob(', dt, callback, args, kwargs)
+    Log('ScheduleJob(', dt, callback, args, kwargs)
     jobID = GetRandomID(length=8)
     # https://apscheduler.readthedocs.io/en/stable/modules/schedulers/base.html#apscheduler.schedulers.base.BaseScheduler.add_job
 
@@ -1522,22 +1497,24 @@ def ScheduleIntervalJob(
         callback,
         *args,
         startDT=None,  # None means datetime.now() + interval
-        timezone='Eastern Standard Time',
         weeks=0,
         days=0,
         hours=0,
         minutes=0,
         seconds=0,
+        name=None,
         **kwargs
 ):
     '''
     Schedule a recurring job
     :return:
     '''
+    Log('ScheduleIntervalJob(', callback, args, kwargs, name, startDT)
     jobID = GetRandomID(length=8)
     # https://apscheduler.readthedocs.io/en/stable/modules/schedulers/base.html#apscheduler.schedulers.base.BaseScheduler.add_job
     app.apscheduler.add_job(
         id=jobID,
+        name=name,
         func=callback,
         trigger='interval',
         start_date=startDT,
